// AsyncSocket.h
// Class implementing normal socket communication

enum AsyncSocketFlags
{
    kDidCompleteOpenForRead  = 1 <<  0,  // If set, open callback has been called for read stream
    kDidCompleteOpenForWrite = 1 <<  1,  // If set, open callback has been called for write stream
    kSocketCanAcceptBytes    = 1 <<  2,  // If set, we know socket can accept bytes. If unset, it's unknown.
    kSocketHasBytesAvailable = 1 <<  3,  // If set, we know socket has bytes available. If unset, it's unknown.
    kStartingHttpHandshake   = 1 <<  4,  // // If set, we're waiting for HTTP negotiation to complete
    kHttpRequestSent         = 1 <<  5,  // // If set, we're waiting for HTTP negotiation to complete
    kHttpAuthRequired        = 1 <<  6,  // // If set, we're waiting for HTTP negotiation to complete
    kHttpAuthCancelled       = 1 <<  7,  // // If set, the user has cancelled the authentication,
                                                //by pressing the cancel key of the Enter Credentials dialog
};

@protocol AsyncSocketDelegate<NSObject>
@required

- ( void )
readEventType: (CFStreamEventType) evntType;

- ( void )
writeEventType: (CFStreamEventType) evntType;

@optional
- (void) proxyAuthenticationCancelled;

@end


@interface AsyncSocket: NSObject
{
    BOOL                m_print;

@protected
    CFWriteStreamRef    m_writeStream;
    CFReadStreamRef        m_readStream;
    NSRunLoop    *        m_runLoop;
    NSRunLoop    *        m_runLoopWrite;

    NSRunLoop    *        m_runLoopStore;
    NSRunLoop    *        m_runLoopWriteStore;

    NSString*           _host;
    unsigned short      _port;
    BOOL                _ssl;
    id<AsyncSocketDelegate> _asdelegate;
    UInt16              _flags;

    NSRecursiveLock*    m_writeLock;
    NSRecursiveLock*    m_readLock;

    BOOL m_isReadStreamCloseRequested;
    BOOL m_isWriteStreamCloseRequested;
    
    int m_errorCode;
}

@property BOOL print;

- ( id )
initWithHost:    ( NSString*         ) host
port:            ( unsigned short ) port
ssl:            ( BOOL           ) ssl
runLoop:        ( NSRunLoop*     ) runLoop
context:        ( void *         ) context;

- ( id )
initWithHost:    ( NSString*         ) host
port:            ( unsigned short ) port
ssl:            ( BOOL           ) ssl
runLoop:        ( NSRunLoop*     ) runLoop
runLoopWrite:   ( NSRunLoop*     ) runLoopWrite
context:        ( void *         ) context;

- (int) connect;

- (void) startTLS: (NSString*) peerHost;

- ( int )
sendData: ( const void * ) buffer
length: ( size_t ) length;

- (BOOL)
sendAllData:(const void *) buffer
length: (size_t) length;

- ( int )
receiveData: ( void* ) buffer
length: ( size_t ) length;

- ( BOOL )
hasData;

- (void) closeAndReleaseStreams;

- ( void )
close;

- (void) closeWriteStream;

- (void) closeReadStream;

- ( BOOL )
isConnected;

- ( BOOL )
isReadyForWriting;

- ( NSString* )
readStreamErrorMessage;

- ( NSString* )
writeStreamErrorMessage;

- ( int )
errorCode;

@end
